"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const assert = require("assert");
const child_process_1 = require("child_process");
const CircularJSON = require("circular-json");
const debug = require("debug");
const Mocha = require("mocha");
const path_1 = require("path");
const runner_1 = require("./runner");
const task_manager_1 = require("./task-manager");
const util_1 = require("./util");
const config_1 = require("../config");
const debugLog = debug('mocha-parallel-tests');
class MochaWrapper extends Mocha {
    constructor() {
        super(...arguments);
        this.isTypescriptRunMode = false;
        this.requires = [];
        this.compilers = [];
    }
    setTypescriptRunMode() {
        this.isTypescriptRunMode = true;
    }
    /**
     * All `--require` options should be applied for subprocesses
     */
    addRequiresForSubprocess(requires) {
        this.requires = requires;
    }
    /**
     * All `--compiler` options should be applied for subprocesses
     */
    addCompilersForSubprocess(compilers) {
        this.compilers = compilers;
    }
    setMaxParallel(maxParallel) {
        this.maxParallel = maxParallel;
    }
    run(onComplete) {
        const { asyncOnly, ignoreLeaks, forbidOnly, forbidPending, fullStackTrace, hasOnly, } = this.options;
        const rootSuite = this.suite;
        const runner = new runner_1.default(rootSuite);
        runner.ignoreLeaks = ignoreLeaks !== false;
        runner.forbidOnly = forbidOnly;
        runner.forbidPending = forbidPending;
        runner.hasOnly = hasOnly;
        runner.fullStackTrace = fullStackTrace;
        runner.asyncOnly = asyncOnly;
        const taskManager = new task_manager_1.default(this.maxParallel);
        for (const file of this.files) {
            const task = () => this.spawnTestProcess(file);
            taskManager.add(task);
        }
        this.options.files = this.files;
        // Refer to mocha lib/mocha.js run() method for more info here
        // @ts-ignore
        const reporter = new this._reporter(runner, 
        // @ts-ignore
        this.options);
        // emit `start` and `suite` events
        // so that reporters can record the start time
        runner.emitStartEvents();
        taskManager.run().then((res) => {
            const retriedTests = [];
            // merge data from subprocess tests into the root suite
            for (const testTesult of res) {
                this.addSubprocessSuites(testTesult);
                retriedTests.push(...this.extractSubprocessRetriedTests(testTesult));
            }
            const done = (failures) => {
                if (reporter.done) {
                    reporter.done(failures, onComplete);
                }
                else if (onComplete) {
                    onComplete(failures);
                }
            };
            // re-emit events
            runner.setTestResults(res, retriedTests, done);
        });
        return runner;
    }
    findRootData(testArtifacts) {
        const endRunnerMessage = testArtifacts.output.find(({ event, type }) => {
            return type === 'runner' && event === 'end';
        });
        assert(endRunnerMessage, `Subprocess ${testArtifacts.file}{${testArtifacts.suiteIndex}} didn't send an "end" message`);
        return endRunnerMessage.data;
    }
    addSubprocessSuites(testArtifacts) {
        const rootSuite = this.suite;
        const serialized = this.findRootData(testArtifacts);
        const { rootSuite: testRootSuite } = CircularJSON.parse(serialized.results, util_1.subprocessParseReviver);
        Object.assign(testRootSuite, {
            parent: rootSuite,
            root: false,
        });
        rootSuite.suites.push(testRootSuite);
    }
    extractSubprocessRetriedTests(testArtifacts) {
        const serialized = this.findRootData(testArtifacts);
        const { retriesTests } = CircularJSON.parse(serialized.retries, util_1.subprocessParseReviver);
        return retriesTests;
    }
    spawnTestProcess(file) {
        return new Promise((resolve) => {
            const extension = this.isTypescriptRunMode ? 'ts' : 'js';
            const runnerPath = path_1.resolve(__dirname, `../subprocess/runner.${extension}`);
            const resolvedFilePath = path_1.resolve(file);
            const forkArgs = ['--test', resolvedFilePath];
            for (const option of config_1.SUITE_OWN_OPTIONS) {
                const suiteProp = `_${option}`;
                forkArgs.push(`--${option}`, this.suite[suiteProp]);
            }
            for (const requirePath of this.requires) {
                forkArgs.push('--require', requirePath);
            }
            for (const compilerPath of this.compilers) {
                forkArgs.push('--compilers', compilerPath);
            }
            if (this.options.delay) {
                forkArgs.push('--delay');
            }
            const test = child_process_1.fork(runnerPath, forkArgs, {
                // otherwise `--inspect-brk` and other params will be passed to subprocess
                execArgv: process.execArgv.filter(util_1.removeDebugArgs),
                stdio: ['ipc'],
            });
            const executable = this.isTypescriptRunMode ? 'ts-node' : 'node';
            debugLog('Process spawned. You can run it manually with this command:');
            debugLog(`${executable} ${runnerPath} ${forkArgs.concat([config_1.DEBUG_SUBPROCESS.argument]).join(' ')}`);
            const output = [];
            const startedAt = Date.now();
            test.on('message', function onMessageHandler({ event, data }) {
                output.push({
                    data,
                    event,
                    type: 'runner',
                });
            });
            test.stdout.on('data', function onStdoutData(data) {
                output.push({
                    data,
                    event: undefined,
                    type: 'stdout',
                });
            });
            test.stderr.on('data', function onStderrData(data) {
                output.push({
                    data,
                    event: undefined,
                    type: 'stderr',
                });
            });
            test.on('close', (code) => {
                debugLog(`Runner exited with code ${code}`);
                resolve({
                    execTime: Date.now() - startedAt,
                    file,
                    output,
                });
            });
        });
    }
}
exports.default = MochaWrapper;
//# sourceMappingURL=mocha.js.map