"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const CircularJSON = require("circular-json");
const Mocha = require("mocha");
const yargs = require("yargs");
const config_1 = require("../config");
const interfaces_1 = require("../interfaces");
const config_2 = require("../config");
const util_1 = require("../util");
const argv = yargs
    .boolean('bail')
    .option('compilers', {
    array: true,
    default: [],
})
    .boolean('delay')
    .boolean('enableTimeouts')
    .number('slow')
    .option('require', {
    array: true,
    default: [],
})
    .number('retries')
    .number('timeout')
    .parse(process.argv);
const debugSubprocess = argv[config_1.DEBUG_SUBPROCESS.yargs];
class Reporter extends interfaces_1.BaseReporter {
    constructor(runner) {
        super(runner);
        /**
         * If `--retries N` option is specified runner can emit `test` events
         * multiple times for retried test cases. These test cases do not exist
         * if the final root suite structure, so we need to store them and return
         * to the main process after the end
         */
        this.runningTests = new Set();
        this.onRunnerStart = () => {
            this.notifyParentThroughIPC('start');
        };
        this.onRunnerEnd = () => {
            const rootSuite = this.runner.suite;
            const retriesTests = [...this.runningTests].map((test) => {
                return Object.assign(test, {
                    [config_1.SUBPROCESS_RETRIED_SUITE_ID]: test.parent[config_1.RUNNABLE_IPC_PROP],
                    parent: null,
                });
            });
            this.notifyParentThroughIPC('end', {
                // can't use the root suite because it will not get revived in the master process
                // @see https://github.com/WebReflection/circular-json/issues/44
                results: CircularJSON.stringify({ rootSuite }),
                retries: CircularJSON.stringify({ retriesTests }),
            });
        };
        this.onRunnerSuiteStart = (suite) => {
            const id = util_1.randomId();
            suite[config_1.RUNNABLE_IPC_PROP] = id;
            this.notifyParentThroughIPC('suite', { id });
        };
        this.onRunnerSuiteEnd = (suite) => {
            this.notifyParentThroughIPC('suite end', {
                id: suite[config_1.RUNNABLE_IPC_PROP],
            });
        };
        this.onRunnerWaiting = ( /* rootSuite: ISuite */) => {
            this.notifyParentThroughIPC('waiting');
        };
        this.onTestStart = (test) => {
            const id = util_1.randomId();
            test[config_1.RUNNABLE_IPC_PROP] = id;
            this.runningTests.add(test);
            this.notifyParentThroughIPC('test', { id });
        };
        this.onTestEnd = (test) => {
            this.runningTests.delete(test);
            this.notifyParentThroughIPC('test end', {
                id: test[config_1.RUNNABLE_IPC_PROP],
            });
        };
        this.onRunnerPass = (test) => {
            this.notifyParentThroughIPC('pass', {
                id: test[config_1.RUNNABLE_IPC_PROP],
            });
        };
        this.onRunnerFail = (test, err) => {
            this.notifyParentThroughIPC('fail', {
                err: {
                    message: err.message,
                    name: err.name,
                    stack: err.stack,
                },
                id: test[config_1.RUNNABLE_IPC_PROP],
            });
        };
        this.onRunnerPending = (test) => {
            this.notifyParentThroughIPC('pending', {
                id: test[config_1.RUNNABLE_IPC_PROP],
            });
        };
        this.onRunnerHookStart = (hook) => {
            const id = hook[config_1.RUNNABLE_IPC_PROP] || util_1.randomId();
            hook[config_1.RUNNABLE_IPC_PROP] = id;
            this.notifyParentThroughIPC('hook', { id });
        };
        this.onRunnerHookEnd = (hook) => {
            this.notifyParentThroughIPC('hook end', {
                id: hook[config_1.RUNNABLE_IPC_PROP],
            });
        };
        runner.on('waiting', this.onRunnerWaiting);
        runner.on('start', this.onRunnerStart);
        runner.on('end', this.onRunnerEnd);
        runner.on('suite', this.onRunnerSuiteStart);
        runner.on('suite end', this.onRunnerSuiteEnd);
        runner.on('test', this.onTestStart);
        runner.on('test end', this.onTestEnd);
        runner.on('pass', this.onRunnerPass);
        runner.on('fail', this.onRunnerFail);
        runner.on('pending', this.onRunnerPending);
        runner.on('hook', this.onRunnerHookStart);
        runner.on('hook end', this.onRunnerHookEnd);
    }
    notifyParentThroughIPC(event, data = {}) {
        if (debugSubprocess) {
            // tslint:disable-next-line:no-console
            console.log({ event, data });
        }
        else {
            process.send({ event, data });
        }
    }
}
const mocha = new Mocha();
mocha.addFile(argv.test);
// --compilers
util_1.applyCompilers(argv.compilers);
// --delay
util_1.applyDelay(mocha, argv.delay);
// --enableTimeouts
util_1.applyNoTimeouts(mocha, argv.enableTimeouts);
// --require
util_1.applyRequires(argv.require);
// --timeout
util_1.applyTimeouts(mocha, argv.timeout);
// apply main process root suite properties
for (const option of config_2.SUITE_OWN_OPTIONS) {
    const suiteProp = `_${option}`;
    mocha.suite[suiteProp] = argv[option];
}
mocha.reporter(Reporter).run();
//# sourceMappingURL=runner.js.map